/*
 *  linux/arch/arm/mach-psionw/psionw.c
 *
 *  Copyright (C) 2001 Tony Lindgren <tony@atomide.com>
 *
 *  Based on the mach-sa1100 code, some portions of the code
 *  Copyright (C) 2000 Nicolas Pitre <nico@cam.org>.
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/tty.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/init.h>

#include <asm/elf.h>
#include <asm/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>

#include <asm/mach/arch.h>
#include <asm/hardware/psionw.h>

extern void psionw_init_irq(void);
extern void psionw_map_io(void);

#define SET_BANK(__nr,__start,__size) \
	mi->bank[__nr].start = (__start), \
	mi->bank[__nr].size = (__size), \
	mi->bank[__nr].node = (((unsigned)(__start) - PHYS_OFFSET) >> 27)

static void __init
fixup_psionw(struct machine_desc *desc, struct param_struct *params,
	     char **cmdline, struct meminfo *mi)
{

#ifdef CONFIG_PSIONW_5MX
	SET_BANK(0, 0xc0000000, 8 * 1024 * 1024);
	SET_BANK(1, 0xc1000000, 8 * 1024 * 1024);
	mi->nr_banks = 2;
#elif CONFIG_PSIONW_5MXPRO24MB
	/* Chris Halls <chris.halls@nikocity.de> */
	SET_BANK(0, 0xc0000000, 8 * 1024 * 1024);
	SET_BANK(1, 0xc1000000, 8 * 1024 * 1024);
	SET_BANK(2, 0xd0000000, 4 * 1024 * 1024);
	SET_BANK(3, 0xd0800000, 4 * 1024 * 1024);
	mi->nr_banks = 4;
#elif CONFIG_PSIONW_5MXPRO32MB
	/* Provided by Thilo Hille <thilo@resourcery.de> */
	SET_BANK(0, 0xc0000000, 8 * 1024 * 1024);
	SET_BANK(1, 0xc1000000, 8 * 1024 * 1024);
	SET_BANK(2, 0xd0000000, 8 * 1024 * 1024);
	SET_BANK(3, 0xd1000000, 8 * 1024 * 1024);
	mi->nr_banks = 4;
#elif CONFIG_PSIONW_REVO
	/* This seems to be correct */
	SET_BANK(0, 0xc0000000, 4 * 1024 * 1024);
	SET_BANK(1, 0xc0800000, 4 * 1024 * 1024);
	mi->nr_banks = 2;
#elif CONFIG_PSIONW_REVOPLUS
	/* Provided by Ilmar Kotte <i.f.a.kotte@altavista.net> */
	SET_BANK(0, 0xc0000000, 4 * 1024 * 1024);
	SET_BANK(1, 0xc0800000, 4 * 1024 * 1024);
	SET_BANK(2, 0xd0000000, 4 * 1024 * 1024);
	SET_BANK(3, 0xd0800000, 4 * 1024 * 1024);
	mi->nr_banks = 4;
#else
#error Unknown machine type!
#endif

	ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);

	/* Merge the command line options from kernel and boot loader */
	strcat(*cmdline, " ");
	strcat(*cmdline, params->commandline);
	strcpy(params->commandline, *cmdline);
}

MACHINE_START(PSIONW, "Psion Windermere NEC ARM-710T")
MAINTAINER("Tony Lindgren")
BOOT_MEM(0xc0000000, 0x80000000, 0xdf000000)

/*
 * VIDEO() does not poke a hole to bootmem.
 * See arch/arm/arm/mm/init.c. 
 */
VIDEO(0xc000d000, 0xc000d000 + LCD_MEM_SIZE)

/* Must sync with TEXTADDR */
BOOT_PARAMS(0xc0048000 - (128 * 1024))

FIXUP(fixup_psionw)
MAPIO(psionw_map_io)
INITIRQ(psionw_init_irq)
MACHINE_END
