/*
 *  linux/include/asm-arm/arch-psionw/time.h
 *
 *  Copyright (C) 2001 Tony Lindgren <tony@atomide.com>
 *  Copyright (C) 2001 Yuji Shinokawa <ysh@mob.or.jp>
 *
 *  Based on the clps711x code, some portions of the code
 *  Copyright (C) 2000 Deep Blue Solutions Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 2001/02/22 Modified for Psion 5MX by Tony Lindgren <tony@atomide.com>
 *
 * 15 Mar 2001  Yuji Shinokawa <ysh@mob.or.jp>
 *    Added to call do_set_rtc() in psionw_timer_interrupt() to adjust RTC.
 *    Adjust xtime.tv_sec for CONFIG_ARCH_PSIONW.
 */

#include <asm/system.h>
#include <asm/leds.h>
#include <asm/hardware/psionw.h>

#define RTC_PORT(x)	(PSIONW_BASE + x)

/*
 * IRQ handler for the timer
 */
static void
psionw_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	do_leds();
	do_timer(regs);
	do_set_rtc();
	do_profile(regs);
}

/*
 * Set up timer interrupt, and return the current time in seconds.
 */
extern __inline__ void
setup_timer(void)
{
	unsigned int timer = 0;
	timer = psionw_readl(TC2CTRL);
	timer |= TC_CLKSEL;		/* 512kHz mode */
	timer |= TC_MODE;		/* Periodic mode */
	timer |= TC_ENABLE;
	psionw_writel(timer, TC2CTRL);
	psionw_writel(5119, TC2LOAD);	/* 512kHz / 100Hz - 1 */
	timer_irq.handler = psionw_timer_interrupt;

	/* 
	 * Make irqs happen for the system timer
	 */
	setup_arm_irq(IRQ_TC2OI, &timer_irq);

	xtime.tv_sec = (psionw_readl(RTCDRU) << 16);
	xtime.tv_sec |= (psionw_readl(RTCDRL) & 0xffff);
}

